#include <LiquidCrystal_I2C.h>
#include <dht.h>
#include <IRremote.h>
#include <SoftwareSerial.h>

SoftwareSerial mySerial(12, 13);

#define DHT_PIN A1
#define PIR_PIN 2
#define GAS_APIN A0
#define IR_RECEIVE_PIN 7
#define BUZZ_PIN 3
#define GAS_DPIN 8 
#define GREEN 9
#define BLUE 10
#define RED 11

#define ONE 22
#define TWO 25
#define THREE 13
#define FOUR 12
#define FIVE 24
#define SIX 94
#define SEVEN 8
#define EIGHT 28
#define NINE 90
#define OK 64
#define UP 70
#define DOWN 21
#define LEFT 68
#define RIGHT 67

dht DHT;
int pirState = 0;
bool tempMode = 0;
bool alarm = 0;
int threshold = 250;
int i = 0;
bool msgSent = 0;
int code[4] = {ONE, TWO, THREE, FOUR};
unsigned long countSec = 0;

LiquidCrystal_I2C lcd(0x27,20,4);
IRrecv irReceiver(IR_RECEIVE_PIN);

void setup()
{
  pinMode(RED, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(BLUE, OUTPUT);
  pinMode(BUZZ_PIN, OUTPUT);
  pinMode(GAS_APIN, INPUT);
  pinMode(GAS_DPIN, INPUT);
  pinMode(PIR_PIN, INPUT);
  pinMode(IR_RECEIVE_PIN, INPUT);

  lcd.init();
  lcd.backlight();

  Serial.begin(9600);
  mySerial.begin(9600);

  IrReceiver.begin(IR_RECEIVE_PIN); 

  analogWrite(RED, 255);
  analogWrite(BLUE, 255);
  analogWrite(GREEN, 0);
}

void playTone(int frequency, int duration) {
  int period = 1000000 / frequency;
  int halfPeriod = period / 2;

  for (long elapsed = 0; elapsed < duration * 1000; elapsed += period) {
    digitalWrite(BUZZ_PIN, HIGH);
    delayMicroseconds(halfPeriod);

    digitalWrite(BUZZ_PIN, LOW);
    delayMicroseconds(halfPeriod);
  }
}

void checkCode() {
  int input[4];
  i = 0;
  while (i < 4) {
    if (IrReceiver.decode()) {
      int comm = IrReceiver.decodedIRData.command;
      if (comm != 0) {
        Serial.println(comm);
        input[i] = comm;
        i++;
        delay(500);
      }
      IrReceiver.resume();
    }
  }

  for(int i = 0; i < 4; i++)
    if (code[i] != input[i]) {
      Serial.println("wrong");
      i = 0;
      return;
    }
  alarm = !alarm;
  Serial.println("good");

  Serial.println(alarm);

  if (pirState == 1) {
    Serial.println("alarm off");
    msgSent = 0;
    pirState = 0;
    analogWrite(RED, 255);
    analogWrite(BLUE, 255);
    analogWrite(GREEN, 0);
  }

  if (alarm) {
    analogWrite(RED, 255);
    analogWrite(GREEN, 255); 
    analogWrite(BLUE, 0);
  } else {
    analogWrite(RED, 255);
    analogWrite(BLUE, 255);
    analogWrite(GREEN, 0);
  }
}

void sendSMS() {
  mySerial.println("AT");
  updateSerial();

  mySerial.println("AT+CMGF=1"); 
  updateSerial();
  mySerial.println("AT+CMGS=\"+40766954696\"");
  updateSerial();
  mySerial.print("Miscare detectata in jurul casei!");
  updateSerial();
  mySerial.write(26);

  msgSent = 1;
  Serial.println("SMS Sent");
}

void updateSerial() {
  delay(500);
  while (Serial.available()) 
    mySerial.write(Serial.read());//Forward what Serial received to Software Serial Port

  while(mySerial.available()) 
    Serial.print(mySerial.read());//Forward what Software Serial received to Serial Port

}

void printData() {
  int readData = DHT.read11(DHT_PIN);

  float t = DHT.temperature;
	float h = DHT.humidity;

  lcd.clear();

  lcd.setCursor(0,0);
  lcd.print("Temp: ");
  
  if (tempMode) {
    lcd.print(t);
    lcd.print("C");
  } else {
    lcd.print((t*9.0)/5.0+32.0);
    lcd.print("F");
  }
  lcd.setCursor(0,1);
  lcd.print("Humidity: ");
  lcd.print((int)h);
  lcd.print("%");
}

void checkGasSensor() {
 int value = analogRead(GAS_APIN);

  if(value > threshold) {
    analogWrite(BLUE, 255);
    analogWrite(GREEN, 255);
    analogWrite(RED, 0);
    playTone(250, 1000);
  } else if (!alarm) {
    analogWrite(RED, 255);
    analogWrite(BLUE, 255);
    analogWrite(GREEN, 0);
  }
}

void checkPirSensor() {
  int pir_val = digitalRead(2);

  if (pir_val == HIGH && countSec == 0) {
    countSec = millis();
  } else if (pir_val == LOW) {
    countSec = 0;
    msgSent = 0;
  }
  unsigned long timeNow = millis();

  if (timeNow - countSec >= 1000 && countSec != 0) {
    pirState = 1;
    analogWrite(BLUE, 255);
    analogWrite(GREEN, 255);
    analogWrite(RED, 0);
    playTone(262, 1000);
    Serial.println("Motion Detected");

    if (!msgSent) {
      sendSMS();
      msgSent = 1;
    }
  }
}

void loop()
{
  printData();

  if (IrReceiver.decode()) {
    IrReceiver.resume();
    int comm = IrReceiver.decodedIRData.command;
    if(comm == DOWN || comm == UP)
      tempMode = !tempMode;
    if (comm == OK) {
      Serial.println("check");
      delay(1000);
      checkCode();
    }
  }

  checkGasSensor();
  if (alarm && !msgSent)
    checkPirSensor();

  if (pirState == 1)
    playTone(262, 1000);
}
